/*
 * NAME
 *   Article.c
 * DESCRIPTION
 *   An 'Article' object represents a USENET news article.
 *
 *   An article consists of a header and a body. The header is a set of header 
 *   lines. A header line consists of a name (e.g. "Subject") and a value. The
 *   value is a variable length string. The body of the article is a variable 
 *   length string. The definition of these data structures are in ArticleP.h.
 * COPYRIGHT
 *   Skim - Off-line news reading package optimized for slow lines.
 *   Copyright (C) 1996  Rene W.J. Pijlman
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 * 
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License
 *   along with This program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * VERSION
 *   Skim version 0.8.4.
 */

#include "Article.h"

#include <assert.h>
#include <ctype.h>
#include <string.h>
#include <time.h>

#include "MemAlloc.h"
#include "VarBuf.h"
#include "Set.h"

#include "ArticleP.h"

#define CLASS_NAME "Article"
#include "CheckForDebris.h"

static int CompareHeadersCaseInsensitive(
    void * HeaderLine1,
    void * HeaderLine2 )
{
    assert( HeaderLineIsOK(HeaderLine1) );
    assert( HeaderLineIsOK(HeaderLine2) );

    return strcasecmp( VBAsString(((struct _HeaderLine *)HeaderLine1)->Name),
                       VBAsString(((struct _HeaderLine *)HeaderLine2)->Name));
}

void HeaderLineDestroy( HeaderLine This )
{
    assert( HeaderLineIsOK(This) );

    if ( This != NULL )
    {
	VBDestroy( This->Name );
	VBDestroy( This->Value );
	MemFree( This );
    }
}

HeaderLine HeaderLineCreate( void )
{
    HeaderLine New = MemAlloc( sizeof(struct _HeaderLine) );

    New->Name = VBCreate();
    New->Value = VBCreate();

    assert( HeaderLineIsOK(New) );

    return New;
}


Boolean HeaderLineIsOK( HeaderLine This )
{
    return This != NULL &&
           VBIsOK(This->Name) &&
	   VBIsOK(This->Value);
}


void HeaderLineSet( HeaderLine This, HeaderNameType Name, VarBuf Value )
{
    assert( HeaderLineIsOK(This) );

    VBDestroy( This->Name );
    VBDestroy( This->Value );

    This->Name = VBCreate();
    VBPrintf( This->Name, "%s", Name );

    This->Value = VBClone(Value);
}


Header HeaderCreate( void )
{
    Header New = MemAlloc( sizeof(struct _Header) );

    New->HeaderLines =
        SetCreate( CompareHeadersCaseInsensitive,
                   (SetElementDestroyFunction)HeaderLineDestroy );

    New->ErrorMessages = VBCreate();

    assert( HeaderIsOK(New) );

    return New;
}


void HeaderDestroy( Header This )
{
    assert( HeaderIsOK(This) );

    if ( This != NULL )
    {
	SetDestroy( This->HeaderLines );
	VBDestroy( This->ErrorMessages );

	MemFree( This );
    }
}

Body BodyCreate( void )
{
    Body New = MemAlloc( sizeof(struct _Body) );

    New->Text = VBCreate();

    New->ErrorMessages = VBCreate();

    assert( BodyIsOK(New) );

    return New;
}

void BodyDestroy( Body This )
{
    assert( BodyIsOK(This) );

    if ( This != NULL )
    {
	VBDestroy( This->Text );
	VBDestroy( This->ErrorMessages );
	MemFree( This );
    }
}

Article ArticleCreate( void )
{
    Article New = MemAlloc( sizeof(struct _Article) );

    New->ArticleHeader = HeaderCreate();
    New->ArticleBody = BodyCreate();

    New->ErrorMessages = VBCreate();

    IncrementObjectCount();

    assert( ArticleIsOK(New) );

    return New;
}

void ArticleDestroy( Article This )
{
    assert( ArticleIsOK(This) );

    if ( This != NULL )
    {
	HeaderDestroy(This->ArticleHeader);
	BodyDestroy(This->ArticleBody);
	VBDestroy(This->ErrorMessages);
	MemFree(This);

	DecrementObjectCount();
    }
}


/*
 * Locate a headerline in the header and make it the current element of the 
 * Set of headerlines.
 */
static void FindHeaderLineInHeader( Header This, HeaderNameType Name )
{
    VarBuf Value = VBCreate();
    HeaderLine Dummy = HeaderLineCreate();

    assert( HeaderIsOK(This) );
    assert( Name != NULL );

    HeaderLineSet( Dummy, Name, Value );

    SetFindElement( This->HeaderLines, Dummy );

    HeaderLineDestroy( Dummy );
    VBDestroy( Value );
}


void HeaderSetHeaderLine(
    Header This,
    HeaderNameType HeaderName,
    VarBuf Value )
{
    assert( HeaderIsOK(This) );
    assert( HeaderName != NULL );
    assert( VBIsOK(Value) );

    FindHeaderLineInHeader( This, HeaderName );
    if ( SetElementExists( This->HeaderLines ) )
    {
	HeaderLine Existing = (HeaderLine)SetGetElement( This->HeaderLines );

	HeaderLineSet( Existing, HeaderName, Value );
    }
    else
    {
	HeaderLine New = HeaderLineCreate();

	HeaderLineSet( New, HeaderName, Value );

	SetAddElement( This->HeaderLines, (void *)New );
    }
}


/*
 * Create a new headerline or overwrite the existing headerline.
 */
void ArticleSetHeaderLine(
    Article This, 
    HeaderNameType HeaderName, 
    VarBuf Value )
{
    assert( ArticleIsOK(This) );
    assert( HeaderName != NULL );
    assert( VBIsOK(Value) );

    HeaderSetHeaderLine( This->ArticleHeader, HeaderName, Value );
}


/*
 * The headerlines which are mandatory.
 *
 * According to RFC 1036, HL_MESSAGE_ID and HL_PATH are also mandatory. Adding
 * these header lines is better left to the news server. Skim is an offline
 * newsreader which is frequently used with dynamic IP addresses, so it cannot
 * figure out the correct fully qualified host name we're sending from.
 */
static char * MandatoryHeaderLines[] = {
    HL_DATE,
    HL_LINES,
    HL_FROM,
    HL_NEWSGROUPS,
    HL_SUBJECT,
};

/* Return True if and only if the header line is mandatory. */
Boolean HeaderLineIsMandatory( HeaderNameType HeaderName )
{
    Boolean Mandatory = False;
    int i;

    assert( HeaderName != NULL );

    for ( i = 0;
          !Mandatory &&
	      i < sizeof MandatoryHeaderLines / sizeof MandatoryHeaderLines[0];
	  i++ )
    {
	if ( !strcmp( MandatoryHeaderLines[i], HeaderName ) )
	{
	    Mandatory = True;
	}
    }

    return Mandatory;
}


/*
 * Assign a value to the body. The existing value is replaced.
 */
void BodySetValue( Body This, VarBuf NewValue )
{
    assert( BodyIsOK(This) );
    assert( VBIsOK(NewValue) );

    VBReset( This->Text );
    VBAppendVB( This->Text, NewValue );

    VBRemoveTrailingBlanks( This->Text );
    VBEndWithNewline( This->Text );
}


/*
 * Assign a value to the body of the article. The existing value is replaced.
 */
void ArticleSetBody( Article This, VarBuf NewValue )
{
    assert( ArticleIsOK(This) );
    assert( VBIsOK(NewValue) );

    BodySetValue( This->ArticleBody, NewValue );

    ArticleSynchronizeNumberOfLines( This );
}


/* Add the "Newsgroups:" headerline to the reply on an existing article. */
static void AddNewsGroupsToReply( Article This, Article Original )
{
    VarBuf NewsGroups = NULL;

    assert( ArticleIsOK(This) );
    assert( ArticleIsOK(Original) );

    assert( !ArticleContainsHeaderLine( This, HL_NEWSGROUPS ) );
    assert( ArticleContainsHeaderLine( Original, HL_NEWSGROUPS ) );

    NewsGroups = ArticleGetHeaderLineValue( Original, HL_FOLLOWUP_TO );

    if ( NewsGroups == NULL )
    {
	NewsGroups = ArticleGetHeaderLineValue( Original, HL_NEWSGROUPS );
    }

    assert ( NewsGroups != NULL );

    ArticleSetHeaderLine( This, HL_NEWSGROUPS, NewsGroups );

    VBDestroy( NewsGroups );
}


/* Add the "Subject:" headerline to the reply on an existing article. */
static void AddSubjectToReply( Article This, Article Original )
{
    VarBuf Subject = NULL;
    VarBuf ReSubject = VBCreate();

    assert( ArticleIsOK(This) );
    assert( ArticleIsOK(Original) );
    assert( !ArticleContainsHeaderLine( This, HL_SUBJECT ) );
    assert( ArticleContainsHeaderLine( Original, HL_SUBJECT ) );

    Subject = ArticleGetHeaderLineValue( Original, HL_SUBJECT );
    assert( Subject != NULL );

    if ( strncasecmp( VBAsString(Subject), RE_PREFIX, strlen(RE_PREFIX) ) )
    {
	VBPrintf( ReSubject, "%s %V", RE_PREFIX, Subject );
    }
    else
    { 
	VBAppendVB( ReSubject, Subject );
    }

    ArticleSetHeaderLine( This, HL_SUBJECT, ReSubject );

    VBDestroy(ReSubject);
    VBDestroy(Subject);
}


/* Add the "References:" headerline to the reply on an existing article. */
static void AddReferencesToReply( Article This, Article Original )
{
    VarBuf OriginalReferences = NULL;
    VarBuf NewReferences = VBCreate();
    VarBuf MessageId = NULL;

    assert( ArticleIsOK(This) );
    assert( ArticleIsOK(Original) );

    MessageId = ArticleGetHeaderLineValue( Original, HL_MESSAGE_ID );
    if ( MessageId != NULL )
    {
	OriginalReferences = ArticleGetHeaderLineValue( Original,
	                                                HL_REFERENCES );
	if ( OriginalReferences != NULL )
	{
	    VBPrintf( NewReferences, "%V %V", OriginalReferences, MessageId );
	}
	else
	{
	    VBPrintf( NewReferences, "%V", MessageId );
	}

	ArticleSetHeaderLine( This, HL_REFERENCES, NewReferences );
    }

    VBDestroy(OriginalReferences);
    VBDestroy(NewReferences);
    VBDestroy(MessageId);
}


static void AppendSignature( VarBuf Body, VarBuf Signature )
{
    assert( VBIsOK(Body) );

    if ( Signature != NULL )
    {
	VBPrintf( Body, "\n-- \n%V", Signature );
    }
}


/* 
 * Add the initial body to the reply on an existing article.
 */
static void AddBodyToReply(
    Article New,
    Article Original, 
    VarBuf Signature )
{
    VarBuf OriginalBody = NULL;
    VarBuf NewBody = VBCreate();
    VarBuf From = NULL;
    Boolean OnNewLine;
    char * p;

    assert( ArticleIsOK(New) );
    assert( ArticleIsOK(Original) );
    assert( ArticleContainsHeaderLine( Original, HL_FROM ) );

    OriginalBody = BodyGetValue( Original->ArticleBody );

    From = ArticleGetHeaderLineValue( Original, HL_FROM );
    assert( From != NULL );

    VBPrintf( NewBody, "%V wrote:\n", From );

    for ( OnNewLine = True, p = VBAsString(OriginalBody); *p != '\0'; p++ )
    {
	if ( OnNewLine )
	{
	    VBAppendString( NewBody, QUOTE_INDENT );
	    OnNewLine = False;
	}

	VBAppendCharacter( NewBody, *p );

	if ( *p == '\n' )
	{
	    OnNewLine = True;
	}
    }

    AppendSignature( NewBody, Signature );

    ArticleSetBody( New, NewBody );

    VBDestroy( OriginalBody );
    VBDestroy( NewBody );
    VBDestroy( From );
}


static void AddBodyToNewArticle( Article This, VarBuf Signature )
{
    VarBuf NewBody = VBCreate();

    assert( ArticleIsOK(This) );
    assert( Signature == NULL || VBIsOK(Signature) );

    VBPrintf( NewBody,
              DEFAULT_BODY_FOR_NEW_ARTICLE );

    AppendSignature( NewBody, Signature );

    ArticleSetBody( This, NewBody );

    VBDestroy( NewBody );
}


/*
 * Construct a new article according to the rules for replies in RFC 1036.
 * The caller is responsible for destroying the new article.
 */
Article ArticleConstructReply(
    Article Original,
    VarBuf  From,
    VarBuf  Path,
    VarBuf  ReplyTo,
    VarBuf  Organization,
    VarBuf  Signature,
    VarBuf  X_NewsReader )
{
    Article New = ArticleCreate();

    assert( ArticleIsOKAndComplete(Original) );
    assert( VBIsOK(From) );
    assert( Path == NULL || VBIsOK(Path) );
    assert( ReplyTo == NULL || VBIsOK(ReplyTo) );
    assert( Organization == NULL || VBIsOK(Organization) );
    assert( Signature == NULL || VBIsOK(Signature) );
    assert( X_NewsReader == NULL || VBIsOK(X_NewsReader) );

    ArticlePrepareForPost( New, From, Path, ReplyTo, Organization, 
                           X_NewsReader );

    AddNewsGroupsToReply( New, Original );
    AddSubjectToReply( New, Original );
    AddReferencesToReply( New, Original );

    AddBodyToReply( New, Original, Signature );

    assert( ArticleIsOK(New) );

    return New;
}


/*
 * Create a new article, which starts a new thread (i.e. it is not a reply).
 */
Article ArticleConstructNew(
    VarBuf NewsGroups, 
    VarBuf Subject, 
    VarBuf From, 
    VarBuf Path, 
    VarBuf ReplyTo, 
    VarBuf Organization, 
    VarBuf Signature, 
    VarBuf X_NewsReader )
{
    Article New = ArticleCreate();

    assert( VBIsOK(NewsGroups) );
    assert( VBIsOK(Subject) );
    assert( VBIsOK(From) );
    assert( Path == NULL || VBIsOK(Path) );
    assert( ReplyTo == NULL || VBIsOK(ReplyTo) );
    assert( Organization == NULL || VBIsOK(Organization) );
    assert( Signature == NULL || VBIsOK(Signature) );
    assert( X_NewsReader == NULL || VBIsOK(X_NewsReader) );

    ArticlePrepareForPost( New, From, Path, ReplyTo, Organization, 
                           X_NewsReader );

    assert( NewsGroups != NULL );
    ArticleSetHeaderLine( New, HL_NEWSGROUPS, NewsGroups );

    assert( Subject != NULL );
    ArticleSetHeaderLine( New, HL_SUBJECT, Subject );

    AddBodyToNewArticle( New, Signature );

    return New;
}


/*
 * Every class has an XxxIsOK() operation which does some basic checks on the
 * representation state of the object. Explanatory error messages are stored
 * in the object. If XxxIsOK() returns False, the error messages can be
 * retrieved with the XxxGetErrorMessages() operation.
 */

VarBuf BodyGetErrorMessages( Body This )
{
    assert( This != NULL && This->ErrorMessages != NULL );

    return This->ErrorMessages;
}


VarBuf HeaderGetErrorMessages( Header This )
{
    assert( This != NULL && This->ErrorMessages != NULL );

    return This->ErrorMessages;
}


VarBuf ArticleGetErrorMessages( Article This )
{
    assert( This != NULL && This->ErrorMessages != NULL );

    return This->ErrorMessages;
}


static Boolean HeaderLineIsOKCallback( void * p1, void * p2 )
{
    Boolean OK = True;
    HeaderLine This = p1;
    Header Parent = p2;

    if ( !HeaderLineIsOK( This ) )
    {
	OK = False;
	VBPrintf( Parent->ErrorMessages, "A headerline is not OK" );
    }

    return OK;
}


Boolean HeaderIsOK( Header This )
{
    Boolean OK = True;

    if ( This != NULL )
    {
	if ( This->ErrorMessages != NULL )
	{
	    if ( This->HeaderLines != NULL )
	    {
		if ( !SetForAllElements( This->HeaderLines,
		                         HeaderLineIsOKCallback,
					 This ) )
		{
		    OK = False;
		}
	    }
	    else
	    {
		OK = False;
		SIOPrintf( StandardError, "This->HeaderLines is NULL\n" );
	    }
	}
	else
	{
	    OK = False;
	    SIOPrintf( StandardError, 
	                       "This->ErrorMessages is NULL\n" );
	}
    }
    else
    {
	OK = False;
	SIOPrintf( StandardError, "Header is NULL\n" );
    }

    return OK;
}


Boolean HeaderIsOKAndComplete( Header This )
{
    Boolean OK = HeaderIsOK( This );
    int i;

    if ( OK )
    {
	assert( This != NULL );
	assert( This->ErrorMessages != NULL );

	/* Check if all mandatory header lines are present and not empty. */
	for ( i = 0;
	      i < sizeof MandatoryHeaderLines / sizeof MandatoryHeaderLines[0];
	      i++ )
	{
	    FindHeaderLineInHeader( This,  MandatoryHeaderLines[i] );
	    if ( SetElementExists( This->HeaderLines ) )
	    {
		HeaderLine HeaderLineToCheck =
		    (HeaderLine)SetGetElement( This->HeaderLines );

		if ( HeaderLineToCheck->Value == NULL ||
		     VBSize(HeaderLineToCheck->Value) == 0 )
		{
		    OK = False;
		    VBPrintf( This->ErrorMessages, "Header line %s is empty\n",  
			       MandatoryHeaderLines[i] );
		}
	    }
	    else
	    {
		OK = False;
		VBPrintf( This->ErrorMessages,
			  "Error: header line '%s:' is missing.\n",
			  MandatoryHeaderLines[i] );
	    }
	}
    }

    return OK;
}


Boolean BodyIsOK( Body This )
{
    Boolean OK = True;

    assert( This != NULL );
    assert( This->ErrorMessages != NULL );

    VBReset( This->ErrorMessages );

    if ( This->Text == NULL || VBSize(This->Text) == 0 )
    {
	VBPrintf( This->ErrorMessages, "The body is empty\n" );
    }

    return OK;
}


Boolean ArticleIsOKAndComplete( Article This )
{
    Boolean OK = True;

    if ( ArticleIsOK(This) )
    {
	if ( !HeaderIsOKAndComplete( This->ArticleHeader ) )
	{
	    OK = False;
	    VBAppendVB( This->ErrorMessages, 
			HeaderGetErrorMessages( This->ArticleHeader ) );
	}
    }
    else
    {
	OK = False;
    }

    return OK;
}


Boolean ArticleIsOK( Article This )
{
    Boolean OK = True;

    if ( This != NULL )
    {
        if ( This->ErrorMessages != NULL )
	{
	    VBReset( This->ErrorMessages );

	    if ( This->ArticleHeader != NULL )
	    {
		if ( !HeaderIsOK( This->ArticleHeader ) )
		{
		    OK = False;
		    VBAppendVB( This->ErrorMessages, 
				HeaderGetErrorMessages( This->ArticleHeader ) );
		}
	    }
	    else
	    {
		OK = False;
		VBPrintf( This->ErrorMessages, "Article has no header\n" );
	    }

	    if ( This->ArticleBody != NULL )
	    {
		if ( !BodyIsOK( This->ArticleBody ) )
		{
		    OK = False;
		    VBAppendVB( This->ErrorMessages,
				BodyGetErrorMessages( This->ArticleBody ) );
		}
	    }
	    else
	    {
		OK = False;
		VBPrintf( This->ErrorMessages, "Article has no body\n" );
	    }

	    if ( !OK )
	    {
		VBPrintf( This->ErrorMessages,
			  "Errors in article\n" );
	    }
	}
	else
	{
	    OK = False;
	    SIOPrintf( StandardError, "This->ErrorMessages is NULL\n" );
	}
    }
    else
    {
	OK = False;
	SIOPrintf( StandardError, "Article is NULL\n" );
    }

    return OK;
}

static Boolean LineIsBlank( const char * Line )
{
    Boolean IsBlank = True;
    const char * p;

    assert( Line != NULL );

    for ( p = Line; *p != '\0' && *p != '\n'; p++ )
    {
        if ( !isspace( *p ) )
        {
            IsBlank = False;
        }
    }

    return IsBlank;
}


typedef enum {
    InHeaderName,
    InHeaderValue,
    InBody
} ParserState;


/*
 * Parse the text representation of an article in a VarBuf and fill an Article
 * object. The text representation is assumed to be in RFC 1036 format, except
 * that newlines in the text are represented as '\n' instead of '\r\n'.
 */
void ArticleSetVB( Article This, VarBuf Value )
{
    VarBuf HeaderName = VBCreate();
    VarBuf HeaderValue = VBCreate();
    VarBuf Body = VBCreate();
    ParserState State = InHeaderName;
    char * p;

    assert( ArticleIsOK(This) );
    assert( VBIsOK(Value) );

    for ( p = VBAsString( Value ); *p != '\0'; p++ )
    {
	switch( State )
	{
	    case InHeaderName:
		if ( *p == ':' )
		{
		    VBRemoveTrailingBlanks( HeaderName );
		    VBRemoveLeadingBlanks( HeaderName );
		    State = InHeaderValue;

		    if (*(p+1) == ' ' || *(p+1) == '\t')
		    {
			p++;
		    }
		}
		else
		{
		    VBAppendCharacter( HeaderName, *p );
		}
	        break;

	    case InHeaderValue:
	        if ( *p == '\n' )
		{
		    Boolean EndOfHeaderValue;

		    if ( LineIsBlank(p+1) )
		    {
			EndOfHeaderValue = True;
			State = InBody;

			if ( *(p+1) != '\0' )
			{
			    p++;
			}
		    }
		    else if ( *(p+1) != ' ' && *(p+1) != '\t' )
		    {
		        EndOfHeaderValue = True;
			State = InHeaderName;
		    }
		    else
		    {
			/* Continuation line. */
			EndOfHeaderValue = False;
		    }

		    if ( EndOfHeaderValue )
		    {
			VBRemoveTrailingBlanks( HeaderName );
			VBRemoveLeadingBlanks( HeaderName );

			HeaderSetHeaderLine( This->ArticleHeader,
			                     VBAsString(HeaderName),
					     HeaderValue );
			VBReset( HeaderName );
			VBReset( HeaderValue );
		    }
		}
		else
		{
		    VBAppendCharacter( HeaderValue, *p );
		}
	        break;

	    case InBody:
	        VBAppendCharacter( Body, *p );
	        break;
	}
    }

    if ( State == InBody )
    {
	ArticleSetBody( This, Body );
    }

    VBDestroy( HeaderName );
    VBDestroy( HeaderValue );
    VBDestroy( Body );
}


/*
 * Return a text representation of the article. The format complies with RFC 
 * 1036, except that newlines in the text are represented as '\n' instead of 
 * '\r\n'.
 */
VarBuf ArticleGetVB( Article This )
{
    VarBuf New = VBCreate();
    VarBuf HeaderAsVB = NULL;
    VarBuf BodyAsVB = NULL;

    assert( ArticleIsOKAndComplete( This ) );

    HeaderAsVB = HeaderGetVB( This->ArticleHeader );
    BodyAsVB = BodyGetValue( This->ArticleBody );

    VBPrintf( New, "%V\n%V", HeaderAsVB, BodyAsVB );

    VBDestroy( HeaderAsVB );
    VBDestroy( BodyAsVB );

    return New;
}


static Boolean AppendHeaderLineToHeaderAsVB(
    HeaderLine This,
    void * ClientData )
{
    assert( HeaderLineIsOK(This) );

    VBPrintf( (VarBuf)ClientData, "%V: %V\n", This->Name, This->Value );

    return True;
}


VarBuf HeaderGetVB( Header This )
{
    VarBuf New = VBCreate();

    assert( HeaderIsOK(This) );

    /* Append the regular header lines. */
    (void)SetForAllElements(
             This->HeaderLines,
             (ForAllElementsCallback)AppendHeaderLineToHeaderAsVB, New );

    return New;
}


VarBuf BodyGetValue( Body This )
{
    assert( BodyIsOK(This) );

    return VBClone( This->Text );
}


/* Tell if the header contains the headerline. */
Boolean HeaderContainsHeaderLine( Header This, HeaderNameType HeaderName )
{
    assert( HeaderIsOK(This) );
    assert( HeaderName != NULL );

    FindHeaderLineInHeader( This, HeaderName );

    return SetElementExists( This->HeaderLines );
}


/* Tell if the article contains the headerline. */
Boolean ArticleContainsHeaderLine( Article This, HeaderNameType HeaderName )
{
    assert( ArticleIsOK(This) );
    assert( HeaderName != NULL );

    return HeaderContainsHeaderLine( This->ArticleHeader, HeaderName);
}


/* Return the number of lines in the body. */
Natural BodyNumnerOfLines( Body This )
{
    assert( BodyIsOK(This) );

    return VBNumberOfLines( This->Text );
}


/* Returns NULL if the header line is not present in the article. */
VarBuf ArticleGetHeaderLineValue( Article This, HeaderNameType HeaderName )
{
    assert( ArticleIsOK(This) );
    assert( HeaderName != NULL );

    return HeaderGetHeaderLineValue( This->ArticleHeader, HeaderName);
}


/* Returns NULL if the header line is not present in the header. */
VarBuf HeaderGetHeaderLineValue( Header This, HeaderNameType HeaderName )
{
    HeaderLine RequestedHeaderLine;
    VarBuf HeaderLineValue = NULL;

    assert( HeaderIsOK(This) );
    assert( HeaderName != NULL );

    FindHeaderLineInHeader( This, HeaderName );
    if ( SetElementExists( This->HeaderLines ) )
    {
	RequestedHeaderLine = (HeaderLine)SetGetElement( This->HeaderLines );

	assert( RequestedHeaderLine != NULL );

	HeaderLineValue = VBClone(RequestedHeaderLine->Value);
    }

    return HeaderLineValue;
}


/*
 * Make the number of lines in the "Lines:" header line consistent with the
 * number of lines in the body of the article.
 */
void ArticleSynchronizeNumberOfLines( Article This )
{
    VarBuf NumberOfLines = VBCreate();

    assert( ArticleIsOK(This) );

    VBPrintf( NumberOfLines, "%lu", BodyNumnerOfLines(This->ArticleBody) );

    HeaderSetHeaderLine( This->ArticleHeader, HL_LINES, NumberOfLines );

    VBDestroy(NumberOfLines);
}


/*
 * RFC 1036 requires that the current date is set when the article is 
 * posted, even when it was set when the article was constructed.
 */
void ArticleSetDateToCurrentDate( Article This )
{
    VarBuf CurrentDate = VBCreate();
    char Buffer[BUFFER_SIZE_FOR_DATE];
    time_t CurrentTime = time(NULL);
    size_t ReturnValue;

    assert( ArticleIsOK(This) );

    /*
     * Follow the RFC 1036 format Wdy, DD Mon YY HH:MM:SS TIMEZONE. Well,
     * not quite. We use a 4 digit year format instead. It appears that the
     * RFCs don't formally allow it, but its quite common in USENET messages.
     */
    ReturnValue = strftime( Buffer, BUFFER_SIZE_FOR_DATE,
                   "%a, %d %b %Y %H:%M:%S %Z", localtime(&CurrentTime) );

    /*
     * K&R (ANSI ed.) and the man page on Linux say different things about the
     * return value. The expression below catches both cases.
     */
    if ( ReturnValue <= 0 || ReturnValue >= BUFFER_SIZE_FOR_DATE )
    {
	SIOPrintf( StandardError, "strftime failed\n" );
	exit( EXIT_FAILURE );
    }

    VBPrintf( CurrentDate, "%s", Buffer );

    HeaderSetHeaderLine( This->ArticleHeader, HL_DATE, CurrentDate );

    VBDestroy( CurrentDate );
}


/*
 * This sets the headerlines which cannot be edited by the user.
 * An article can be prepared for post before it is edited (to make the
 * headerlines visible to the user) and before it is posted (to fix
 * headerlines edited by the user). It should not be necessary to prepare
 * an article twice when the user interface prevents the user from editing
 * the read-only headerlines.
 */
void ArticlePrepareForPost(
    Article This,
    VarBuf From,
    VarBuf Path,
    VarBuf ReplyTo,
    VarBuf Organization,
    VarBuf X_NewsReader )
{
    assert(ArticleIsOK(This));

    assert( VBIsOK(From) );
    assert( Path == NULL || VBIsOK(Path) );
    assert( ReplyTo == NULL || VBIsOK(ReplyTo) );
    assert( Organization == NULL || VBIsOK(Organization) );
    assert( X_NewsReader == NULL || VBIsOK(X_NewsReader) );

    ArticleSetHeaderLine( This, HL_FROM, From );

    if ( Path != NULL )
    {
        ArticleSetHeaderLine( This, HL_PATH, Path );
    }

    if ( ReplyTo != NULL )
    {
        ArticleSetHeaderLine( This, HL_REPLY_TO, ReplyTo );
    }

    if ( Organization != NULL )
    {
        ArticleSetHeaderLine( This, HL_ORGANIZATION, Organization );
    }

    if ( X_NewsReader != NULL )
    {
	ArticleSetHeaderLine( This, HL_X_NEWSREADER, X_NewsReader );
    }

    ArticleSetDateToCurrentDate( This );
    ArticleSynchronizeNumberOfLines( This );

    if ( !ArticleIsOK(This) )
    {
	SIOPrintf( StandardError, "Errors in This article:\n%V",
	           ArticleGetErrorMessages(This) );
	exit( EXIT_FAILURE );
    }

    assert( ArticleIsOK(This) );
}
